//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <string>
#include <SDL.h>
#include <sigc++/object_slot.h>
#include "xmlhelper.h"

class Configuration : public SigC::Object
{
    public:
        // CREATORS
        Configuration();
        ~Configuration();

        void loadConfigurationFile(std::string fileName);

        //as the name implies
        static bool s_smoothScrolling;
        static bool s_showNextPlayer;
        
        //testing: two AI players fight each other
        static bool s_testing;

        //the paths
        static std::string s_dataPath;
        static std::string s_savePath;

        //main window and resolution settings
        static int s_width;
        static int s_height;
        static Uint32 s_flags;

        //settings for pixmaps etc. (especially hardware surfaces)
        static Uint32 s_surfaceFlags;

        //the maximum size of the graphics cache
        static Uint32 s_cacheSize;

        //zip and obfuscate save files
        static bool s_zipfiles;

    private:
        bool parseConfiguration(std::string tag, XML_Helper* helper);
};

#endif // CONFIGURATION_H
