//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "GameMap.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include "citylist.h"
#include "city.h"
#include "ruin.h"
#include "temple.h"
#include "playerlist.h"

//#define debug(x) {std::cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<std::endl;}
#define debug(x)

GameMap* GameMap::s_instance = 0;
TileSet* GameMap::s_tileSet = 0;

int GameMap::s_size = 100;

GameMap* GameMap::getInstance()
{
    if (s_instance != 0)
      return s_instance;
    else return 0;
}


GameMap* GameMap::getInstance(std::string TilesetName)
{
    if (s_instance == 0)
    {
        s_instance = new GameMap(TilesetName);

    }
    return s_instance;
}

GameMap* GameMap::getInstance(XML_Helper* helper)
{
    if (s_instance)
        deleteInstance();

    s_instance = new GameMap(helper);

    return s_instance;
}

GameMap* GameMap::getInstance(XML_Helper* helper, std::string TilesetName)
{
    if (s_instance)
        deleteInstance();

    s_instance = new GameMap(helper, TilesetName);

    return s_instance;
}

void GameMap::deleteInstance()
{
    if (s_instance)
        delete s_instance;
    s_instance = 0;
}

GameMap::GameMap(std::string TilesetName)
    :d_name(""), d_info("")
{
    if (!s_tileSet)
        s_tileSet = new TileSet(TilesetName);

    d_map = new Maptile*[s_size*s_size];
    for (int j = 0; j < s_size; j++)
        for (int i = 0; i < s_size; i++)
            d_map[j*s_size + i] = 0;

}

GameMap::GameMap(XML_Helper* helper)
{
	
	std::string types;
	std::string buildings;
	std::string t_name;

	helper->getData(s_size, "size");
	helper->getData(t_name,"tileset");
	helper->getData(types, "types");
	helper->getData(buildings, "buildings");

    if (!s_tileSet)
        s_tileSet = new TileSet(t_name);

    //create the map
    d_map = new Maptile*[s_size*s_size];

	for (int j = 0; j < s_size; j++)
    {
		for (int i = 0; i < s_size; i++)
		{
            //due to the circumstances, types is a long stream of
            //numbers, so read it character for character (no \n's or so)
			char type = types[j*s_size + i];  

            //the chars now hold the ascii representation of the numbers, which
            //we don't want
            type -= '0';
			d_map[j*s_size + i] = new Maptile(s_tileSet, i, j, type);
		}
    }

	for (int j = 0; j < s_size; j++)
    {
		for (int i = 0; i < s_size; i++)
		{
			char buildingTemp = buildings[j*s_size + i];  
            Maptile::Building building = (Maptile::Building)(buildingTemp - '0');
            getTile(i, j)->setBuilding(building);
		}
    }
}

GameMap::GameMap(XML_Helper* helper, std::string TilesetName)
{

	std::string types;
	std::string buildings;
	helper->getData(s_size, "size");
	helper->getData(types, "types");
	//	std::string t_name;
	//helper->getData(tmp,"tileset");
	helper->getData(buildings, "buildings");

	if (!s_tileSet)
	  s_tileSet = new TileSet(TilesetName);

    //create the map
    d_map = new Maptile*[s_size*s_size];

	for (int j = 0; j < s_size; j++)
    {
		for (int i = 0; i < s_size; i++)
		{
            //due to the circumstances, types is a long stream of
            //numbers, so read it character for character (no \n's or so)
			char type = types[j*s_size + i];  

            //the chars now hold the ascii representation of the numbers, which
            //we don't want
            type -= '0';
			d_map[j*s_size + i] = new Maptile(s_tileSet, i, j, type);
		}
    }

	for (int j = 0; j < s_size; j++)
    {
		for (int i = 0; i < s_size; i++)
		{
			char buildingTemp = buildings[j*s_size + i];  
            Maptile::Building building = (Maptile::Building)(buildingTemp - '0');
            getTile(i, j)->setBuilding(building);
		}
    }
}

GameMap::~GameMap()
{
    //delete s_tileSet;
    //see s_tileSet for explanation of commenting this out

	for (int i = 0; i < s_size; i++)
	{
		for (int j = 0; j < s_size; j++)
		{
            if (d_map[j*s_size + i])
                delete d_map[j*s_size + i];
		}
	}

    delete[] d_map;
}

bool GameMap::fill(MapGenerator* generator)
//basically, this does the same as the former random function, but you don't
//need to go the whole way via dumping the map in a file etc.
{
    int size;
    int type;
    char* mymap;

    mymap = generator->getMap(size);

    //the sizes should definitely match, else we have a problem here
    if (size != s_size)
    {
        std::cerr <<"Error in GameMap::fillMap: sizes don't match!! Exiting.\n";
        exit(-1);
    }

    for (int j = 0; j < size; j++)
        for (int i = 0; i < size; i++)
        {
            switch (mymap[i+j*size])
            {
 				case CHARACTER_WATER:
					type = 1;
					break;
				case CHARACTER_FOREST:
					type = 2; 
					break;
				case CHARACTER_HILLS:
					type = 3;
					break;
				case CHARACTER_MOUNTAINS:
					type = 4;
					break;
				case CHARACTER_SWAMP:
					type = 5;
					break;
				default:
					type = 0;
					break;
            }
			d_map[j*s_size + i] = new Maptile(s_tileSet, i, j, type);
        }

    mymap = generator->getBuildings(size);

    for (int j = 0; j < size; j++)
        for (int i = 0; i < size; i++)
        {
            switch (mymap[j*size + i])
            {
                case CHARACTER_CITY_MINOR:
                case CHARACTER_CITY_MAJOR:
                    getTile(i, j)->setMoves(1);
                    getTile(i, j)->setBuilding(Maptile::CITY);
                    break;
                case CHARACTER_RUIN:
                    getTile(i, j)->setBuilding(Maptile::RUIN);
                    break;
                case CHARACTER_TEMPLE:
                    getTile(i, j)->setBuilding(Maptile::TEMPLE);
                    break;
            }
        }
    
    return true;
}

bool GameMap::save(XML_Helper* helper)
{
	bool retval = true;

	std::stringstream types;
    std::stringstream buildings;

	types << endl;
	buildings << endl;
	for (int i = 0; i < s_size; i++)
	{
		for (int j = 0; j < s_size; j++)
		{
			types << getTile(j, i)->getType();
			buildings << getTile(j, i)->getBuilding();
		}
		buildings << endl;
		types << endl;
	}


	retval &= helper->openTag("map");
	retval &= helper->saveData("size", s_size);
	retval &= helper->saveData("tileset", s_tileSet->getName());
	retval &= helper->saveData("types", types.str());
	retval &= helper->saveData("buildings", buildings.str());
	retval &= helper->closeTag();
	return retval;
}

Maptile* GameMap::getTile(int x, int y)
{
    return d_map[y*s_size + x];
}

// End of file
