//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef BIGMAP_H
#define BIGMAP_H

#include <pgeventobject.h>
#include <pgwidget.h>
#include <pgtimerobject.h>
#include "MapRenderer.h"
#include <sigc++/sigc++.h>
#include "stack.h"

// The big map views the map in a more detailed way
// It's the playing area

class BigMap : public PG_EventObject, public PG_Widget, public PG_TimerObject,
    public SigC::Object
{
	public:
		// CREATORS
		BigMap(PG_Widget* parent, PG_Rect rect);
		~BigMap();

		Uint32 eventTimer(PG_TimerID id, Uint32 interval);

		// MANIPULATORS
		void setViewrect(PG_Rect* viewrect);
		void stackSelected();
		void stackDeselected();
		void stackMoved();
		void centerView(const PG_Point& p);
		void setEnable (bool enable){d_enable=enable;}
        void drawSelector(bool changeFrame);

        //Callbacks
        void interruptTimer();
        void restartTimer();
        
        SigC::Signal0<void> schangingViewrect;
        SigC::Signal1<void, Stack*> sselectingStack;
		
	private:
		// EVENT HANDLERS
		void eventDraw(SDL_Surface* surface, const PG_Rect& rect);
		bool eventMouseButtonDown(const SDL_MouseButtonEvent* event);

		// MANIPULATORS
		const PG_Point getRelativePos(const PG_Point& absolutePos); 
        const PG_Point getRelativeXPos(const PG_Point& absolutePos);

        void alignXRect();
        
		// DATA
        MapRenderer* d_renderer;
		PG_Rect* d_viewrect;
        
        PG_Rect d_oldrect;
        PG_Rect d_xrect;
        SDL_Surface* d_xscreen;
        int d_scrollstat;
        bool d_selupdate;
        
        SDL_Surface* d_razedcitypic;
		SDL_Surface* d_arrows;
		SDL_Surface* d_ruinpic;
		SDL_Surface* d_templepic;
		SDL_Surface* d_swordpic;
		SDL_Surface* d_selector[2];
		SDL_Surface* d_flagpic[9][8];   // (8 player + neutral) * 8 flagssizes
		SDL_Surface* d_polepic;		
        
        Uint32 d_timerID;
		bool d_enable;
};

#endif // BIGMAP_H

// End of file
