//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef REAL_PLAYER_H
#define REAL_PLAYER_H

#include <string>
#include <list>
#include <pgmessageobject.h>
#include "defs.h"
#include "player.h"

#include "signals.h" // include culpritsType

// This is the base class for all local players who actually do something on
// their own (in difference to network players who just follow a chain of orders)
// See player.h for further details. AI players are derived from this class, but
// it is also used as is for human players

class RealPlayer : public Player
{
	public:
		// CREATORS
		RealPlayer(std::string name, std::string armyset, SDL_Color color);
		RealPlayer(std::string name, std::string armyset, SDL_Color color, Player::TYPE type);
		RealPlayer(const Player&);
		RealPlayer(XML_Helper* helper);
		~RealPlayer();
		
		//ACTIONS
		virtual bool startTurn();
        virtual bool invadeCity(City* c);
        virtual bool recruitHero(Hero* hero, int cost);
        
        bool stackSplit(Stack* s);
        bool stackJoin(Stack* receiver, Stack* joining);

        bool stackMove(Stack* s);
        bool stackMove(Stack* s, PG_Point dest, bool follow);
        bool stackSearchRuin(Stack* s, Ruin* r);
        bool stackVisitTemple(Stack* s, Temple* t);
        
        Fight::RESULT stackFight(Stack* sattacker, Stack* sdefender);
		Fight::RESULT stackFightCity(std::vector<Stack*> attackers,
						City* target);

        Fight::RESULT coreStackFight(const char *context, 
                                     std::vector<Stack*>& attackers,
                                     std::vector<Stack*>& defenders,
                                     void *client_data = NULL);

        double getDefenderXp() {return d_defender_xp;};
        double getAttackerXp()   {return d_attacker_xp;};
		//double setFighterXp(vector<Stack*>& stacks);
        void updateArmyXp(std::vector<Stack*>& stacks,bool emit,double xp_sum);

        void getHeroes(std::vector<Stack*>& stacks, culpritsType& dst);
        double removeDeadArmies(std::vector<Stack*>& stacks,
                              culpritsType& culprits);

		bool cityOccupy(City* c);
        bool cityPillage(City* c);
		bool cityRaze(City* c);
		bool cityUpgradeDefense(City* c);
		bool cityBuyProduction(City* c, int armytype);
		bool cityChangeProduction(City* c, int production);
		
	protected:
        //if reward != 0, give this reward, else randomize
		bool giveReward (int level, Stack* s, fl_Treasure* reward = 0);
		bool stackMoveOneStep(Stack* s);
        double d_attacker_xp;
		double d_defender_xp;
};

#endif // REAL_PLAYER_H

// End of file
