//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef SMALLMAP_H
#define SMALLMAP_H

#include <pgeventobject.h>
#include <pgwidget.h>
#include <pgtimerobject.h>
#include <sigc++/sigc++.h>

class SmallMap : public PG_EventObject, public PG_Widget, public PG_TimerObject,
                public SigC::Object
{
	public:
		// CREATORS
		SmallMap(PG_Widget* parent, PG_Rect rect,
				int x_squares, int y_squares);
		~SmallMap();

		// MANIPULATORS
		void stackMoved();
		void drawViewrect();
		
		Uint32 eventTimer(PG_TimerID id, Uint32 interval);
        void eventMouseLeave(){d_pressed = false;}

		// ACCESSORS
		PG_Rect* getViewrect(){return d_viewrect;}

		// CALLBACKS
		PARAGUI_CALLBACK(changedViewrect);
        void interruptTimer();
        void restartTimer();

        SigC::Signal0<void> schangingViewrect;
        
	private:
		void setViewrect(int x, int y);

		// EVENT HANDLERS
		void eventDraw(SDL_Surface* surface, const PG_Rect& rect);
		bool eventMouseButtonDown(const SDL_MouseButtonEvent* event);
		bool eventMouseButtonUp(const SDL_MouseButtonEvent* event);
		bool eventMouseMotion(const SDL_MouseMotionEvent* event);

		// DATA
		SDL_Surface* d_staticMap;
		PG_Rect* d_viewrect;
		bool d_pressed;
		int d_r;
		int d_g;
		int d_b;
		bool d_add;
        Uint32 d_timerID;
};

#endif // SMALLMAP_H
